@@ -3,7 +3,7 @@
 #
 # === This file is part of Calamares - <https://calamares.io> ===
 #
-#   SPDX-FileCopyrightText: 2014-2018 Philip Müller <philm@manjaro.org>
+#   SPDX-FileCopyrightText: 2014-2022 Philip Müller <philm@manjaro.org>
 #   SPDX-FileCopyrightText: 2014-2015 Teo Mrnjavac <teo@kde.org>
 #   SPDX-FileCopyrightText: 2014 Kevin Kofler <kevin.kofler@chello.at>
 #   SPDX-FileCopyrightText: 2017 Alf Gaida <agaida@siduction.org>
@@ -163,6 +163,8 @@ class DesktopEnvironment:
 # /usr/local/bin) then the DE is assumed to be installed
 # and to use that .desktop filename.
 desktop_environments = [
+    DesktopEnvironment('/usr/bin/startplasmamobile', 'plasma-mobile'),  # KDE Plasma Mobile 5.26+
+    DesktopEnvironment('/usr/bin/kwinwrapper', 'plasma-mobile'),  # KDE Plasma Mobile
     DesktopEnvironment('/usr/bin/startplasma-x11', 'plasma'),  # KDE Plasma 5.17+
     DesktopEnvironment('/usr/bin/startkde', 'plasma'),  # KDE Plasma 5
     DesktopEnvironment('/usr/bin/startkde', 'kde-plasma'),  # KDE Plasma 4
@@ -197,6 +199,8 @@ desktop_environments = [
     DesktopEnvironment('/usr/bin/sway', 'sway'),
     DesktopEnvironment('/usr/bin/ukui-session', 'ukui'),
     DesktopEnvironment('/usr/bin/cutefish-session', 'cutefish-xsession'),
+    DesktopEnvironment('/usr/bin/river', 'river'),
+    DesktopEnvironment('/usr/bin/Hyprland', 'hyprland'),
 ]
 
 
@@ -374,14 +378,30 @@ class DMgdm(DisplayManager):
         GDM exists with different executable names, so search
         for one of them and use it.
         """
-        for executable, config in (
+        candidates = (
             ( "gdm", "etc/gdm/custom.conf" ),
-            ( "gdm3", "etc/gdm3/daemon.conf" )
-        ):
+            ( "gdm3", "etc/gdm3/daemon.conf" ),
+            ( "gdm3", "etc/gdm3/custom.conf" ),
+        )
+
+        def have_executable(executable : str):
             bin_path = "{!s}/usr/bin/{!s}".format(self.root_mount_point, executable)
             sbin_path = "{!s}/usr/sbin/{!s}".format(self.root_mount_point, executable)
-            if os.path.exists(bin_path) or os.path.exists(sbin_path):
-                # Keep the found-executable name around for later
+            return os.path.exists(bin_path) or os.path.exists(sbin_path)
+
+        def have_config(config : str):
+            config_path = "{!s}/{!s}".format(self.root_mount_point, config)
+            return os.path.exists(config_path)
+
+        # Look for an existing configuration file as a hint, then
+        # keep the found-executable name and config around for later.
+        for executable, config in candidates:
+            if have_config(config) and have_executable(executable):
+                self.executable = executable
+                self.config = config
+                return True
+        for executable, config in candidates:
+            if have_executable(executable):
                 self.executable = executable
                 self.config = config
                 return True
@@ -444,7 +464,6 @@ class DMgdm(DisplayManager):
 
                     userfile.write("Icon=\n")
 
-
     def basic_setup(self):
         if libcalamares.utils.target_env_call(
                 ['getent', 'group', 'gdm']
@@ -481,84 +500,6 @@ class DMgdm(DisplayManager):
         pass
 
 
-class DMkdm(DisplayManager):
-    name = "kdm"
-    executable = "kdm"
-
-    def set_autologin(self, username, do_autologin, default_desktop_environment):
-        # Systems with KDM as Desktop Manager
-        kdm_conf_path = os.path.join(
-            self.root_mount_point, "usr/share/config/kdm/kdmrc"
-            )
-        # Check which path is in use: SUSE does something else.
-        # Also double-check the default setting. Pick the first
-        # one that exists in the target.
-        for candidate_kdmrc in (
-            "usr/share/config/kdm/kdmrc",
-            "usr/share/kde4/config/kdm/kdmrc",
-        ):
-            p = os.path.join(self.root_mount_point, candidate_kdmrc)
-            if os.path.exists(p):
-                kdm_conf_path = p
-                break
-        text = []
-
-        if os.path.exists(kdm_conf_path):
-            with open(kdm_conf_path, 'r') as kdm_conf:
-                text = kdm_conf.readlines()
-
-            with open(kdm_conf_path, 'w') as kdm_conf:
-                for line in text:
-                    if 'AutoLoginEnable=' in line:
-                        if do_autologin:
-                            line = 'AutoLoginEnable=true\n'
-                        else:
-                            line = 'AutoLoginEnable=false\n'
-
-                    if do_autologin and 'AutoLoginUser=' in line:
-                        line = "AutoLoginUser={!s}\n".format(username)
-
-                    kdm_conf.write(line)
-        else:
-            return (
-                _("Cannot write KDM configuration file"),
-                _("KDM config file {!s} does not exist").format(kdm_conf_path)
-                )
-
-    def basic_setup(self):
-        if libcalamares.utils.target_env_call(
-                ['getent', 'group', 'kdm']
-                ) != 0:
-            libcalamares.utils.target_env_call(
-                ['groupadd', '-g', '135', 'kdm']
-                )
-
-        if libcalamares.utils.target_env_call(
-                ['getent', 'passwd', 'kdm']
-                ) != 0:
-            libcalamares.utils.target_env_call(
-                ['useradd',
-                    '-u', '135',
-                    '-g', 'kdm',
-                    '-d', '/var/lib/kdm',
-                    '-s', '/bin/false',
-                    '-r',
-                    '-M',
-                    'kdm'
-                    ]
-                )
-
-        libcalamares.utils.target_env_call(
-            ['chown', '-R', '135:135', 'var/lib/kdm']
-            )
-
-    def desktop_environment_setup(self, desktop_environment):
-        pass
-
-    def greeter_setup(self):
-        pass
-
-
 class DMlxdm(DisplayManager):
     name = "lxdm"
     executable = "lxdm"
@@ -622,6 +563,11 @@ class DMlightdm(DisplayManager):
     name = "lightdm"
     executable = "lightdm"
 
+    # Can be overridden in the .conf file. With no value it won't match any
+    # desktop file in the xgreeters directory and instead we end up picking
+    # the alphabetically first file there.
+    preferred_greeters = []
+
     def set_autologin(self, username, do_autologin, default_desktop_environment):
         # Systems with LightDM as Desktop Manager
         # Ideally, we should use configparser for the ini conf file,
@@ -671,7 +617,6 @@ class DMlightdm(DisplayManager):
                     _("LightDM config file {!s} does not exist").format(lightdm_conf_path)
                     )
 
-
     def basic_setup(self):
         libcalamares.utils.target_env_call(
             ['mkdir', '-p', '/run/lightdm']
@@ -711,40 +656,52 @@ class DMlightdm(DisplayManager):
                 )
             )
 
+    def find_preferred_greeter(self):
+        """
+        On Debian, lightdm-greeter.desktop is typically a symlink managed
+        by update-alternatives pointing to /etc/alternatives/lightdm-greeter
+        which is also a symlink to a real .desktop file back in /usr/share/xgreeters/
+
+        Returns a path *into the mounted target* of the preferred greeter -- usually
+        a .desktop file that specifies where the actual executable is. May return
+        None to indicate nothing-was-found.
+        """
+        greeters_dir = "usr/share/xgreeters"
+        greeters_target_path = os.path.join(self.root_mount_point, greeters_dir)
+        available_names = os.listdir(greeters_target_path)
+        available_names.sort()
+        desktop_names = [n for n in self.preferred_greeters if n in available_names] # Preferred ones
+        if desktop_names:
+            return desktop_names[0]
+        desktop_names = [n for n in available_names if n.endswith(".desktop")] # .. otherwise any .desktop
+        if desktop_names:
+            return desktop_names[0]
+        return None
+
     def greeter_setup(self):
-        lightdm_conf_path = os.path.join(
-            self.root_mount_point, "etc/lightdm/lightdm.conf"
-            )
+        lightdm_conf_path = os.path.join(self.root_mount_point, "etc/lightdm/lightdm.conf")
+        greeter_name = self.find_preferred_greeter()
 
-        # configure lightdm-greeter
-        greeter_path = os.path.join(
-            self.root_mount_point, "usr/share/xgreeters"
-            )
+        if greeter_name is not None:
+            greeter = os.path.basename(greeter_name)  # Follow symlinks, hope they are not absolute
+            if greeter.endswith('.desktop'):
+                greeter = greeter[:-8]  # Remove ".desktop" from end
 
-        if (os.path.exists(greeter_path)):
-            # configure first found lightdm-greeter
-            for entry in os.listdir(greeter_path):
-                if entry.endswith('.desktop'):
-                    greeter = entry.split('.')[0]
-                    libcalamares.utils.debug(
-                        "found greeter {!s}".format(greeter)
-                        )
-                    os.system(
-                        "sed -i -e \"s/^.*greeter-session=.*"
-                        "/greeter-session={!s}/\" {!s}".format(
-                            greeter,
-                            lightdm_conf_path
-                            )
-                        )
-                    libcalamares.utils.debug(
-                        "{!s} configured as greeter.".format(greeter)
-                        )
-                    break
-            else:
-                return (
-                    _("Cannot configure LightDM"),
-                    _("No LightDM greeter installed.")
-                    )
+            libcalamares.utils.debug("found greeter {!s}".format(greeter))
+            os.system(
+                "sed -i -e \"s/^.*greeter-session=.*"
+                "/greeter-session={!s}/\" {!s}".format(
+                    greeter,
+                    lightdm_conf_path
+                )
+            )
+            libcalamares.utils.debug("{!s} configured as greeter.".format(greeter))
+        else:
+            libcalamares.utils.error("No greeter found at all, preferred {!s}".format(self.preferred_greeters))
+            return (
+                _("Cannot configure LightDM"),
+                _("No LightDM greeter installed.")
+            )
 
 
 class DMslim(DisplayManager):
@@ -793,11 +750,13 @@ class DMsddm(DisplayManager):
     name = "sddm"
     executable = "sddm"
 
+    configuration_file = "/etc/sddm.conf"
+
     def set_autologin(self, username, do_autologin, default_desktop_environment):
         import configparser
 
         # Systems with Sddm as Desktop Manager
-        sddm_conf_path = os.path.join(self.root_mount_point, "etc/sddm.conf.d/kde_settings.conf")
+        sddm_conf_path = os.path.join(self.root_mount_point, self.configuration_file.lstrip('/'))
 
         sddm_config = configparser.ConfigParser(strict=False)
         # Make everything case sensitive
@@ -840,6 +799,7 @@ class DMgreetd(DisplayManager):
     executable = "greetd"
     greeter_user = "greeter"
     greeter_group = "greetd"
+    greeter_css_location = None
     config_data = {}
 
     def os_path(self, path):
@@ -895,7 +855,7 @@ class DMgreetd(DisplayManager):
 
     def desktop_environment_setup(self, default_desktop_environment):
         with open(self.environments_path(), 'w') as envs_file:
-            envs_file.write(default_desktop_environment.desktop_file)
+            envs_file.write(default_desktop_environment.executable)
             envs_file.write("\n")
 
     def greeter_setup(self):
@@ -905,8 +865,10 @@ class DMgreetd(DisplayManager):
         self.config_load()
 
         de_command = default_desktop_environment.executable
-        if os.path.exists(self.os_path("usr/bin/gtkgreed")) and os.path.exists(self.os_path("usr/bin/cage")):
-            self.config_data['default_session']['command'] = "cage -s -- gtkgreet"
+        if os.path.exists(self.os_path("usr/bin/gtkgreet")) and os.path.exists(self.os_path("usr/bin/cage")):
+            self.config_data['default_session']['command'] = "cage -d -s -- gtkgreet"
+            if self.greeter_css_location:
+                self.config_data['default_session']['command'] += f" -s {self.greeter_css_location}"
         elif os.path.exists(self.os_path("usr/bin/tuigreet")):
             tuigreet_base_cmd = "tuigreet --remember --time --issue --asterisks --cmd "
             self.config_data['default_session']['command'] = tuigreet_base_cmd + de_command
@@ -915,8 +877,12 @@ class DMgreetd(DisplayManager):
         else:
             self.config_data['default_session']['command'] = "agreety --cmd " + de_command
 
-        if do_autologin == True:
+        if do_autologin:
+            # Log in as user, with given DE
             self.config_data['initial_session'] = dict(command = de_command, user = username)
+        elif 'initial_session' in self.config_data:
+            # No autologin, remove any autologin that was copied from the live ISO
+            del self.config_data['initial_session']
 
         self.config_write()
 
@@ -1057,6 +1023,11 @@ def run():
     # Do the actual configuration and collect messages
     dm_setup_message = []
     for dm in dm_impl:
+        dm_specific_configuration = libcalamares.job.configuration.get(dm.name, None)
+        if dm_specific_configuration and isinstance(dm_specific_configuration, dict):
+            for k, v in dm_specific_configuration.items():
+                if hasattr(dm, k):
+                    setattr(dm, k, v)
         dm_message = None
         if enable_basic_setup:
             dm_message = dm.basic_setup()
